import {tap} from 'rxjs/operators';
import { Injectable } from '@angular/core';
import {
  Router,
  CanActivate,
  ActivatedRouteSnapshot,
  RouterStateSnapshot,
  CanLoad,
  Route, UrlSegment
} from '@angular/router';
import {Observable, Subscription} from 'rxjs';
import {CommunityService} from '../community/community.service';
import { EnvironmentSpecificService} from '../../utils/properties/environment-specific.service';
import {ConnectHelper} from '../connectHelper';
import {properties} from "../../../../environments/environment";

@Injectable()
export class ConnectRIGuard implements  CanActivate, CanLoad {
  sub: Subscription = null;

  constructor(private router: Router,
              private communityService: CommunityService,
              private propertiesService: EnvironmentSpecificService) {
  }

  check(community: string): Observable<boolean> | boolean {
    return this.communityService.isRITypeByState(properties, properties['communityAPI'] + community).pipe(tap(authorized => {
      if (!authorized) {
        this.router.navigate(['errorcommunity']);
      }
    }));
  }
  canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): Observable<boolean> | boolean {
    return this.check(route.queryParams['communityId']);
  }

  canLoad(route: Route, segments: UrlSegment[]): Observable<boolean> | Promise<boolean> | boolean {
    const path = '/' + route.path + document.location.search;
    return this.check(ConnectHelper.getCommunityFromPath(path));
  }

  canDeactivate() {
    if(this.sub) {
      this.sub.unsubscribe();
    }
    return true;
  }
}
