import { Component, ViewChild, ElementRef } from '@angular/core';
import { ActivatedRoute, Router } from "@angular/router";
import { DivContentFormComponent } from "./div-help-content-form.component";
import { DivHelpContent } from "../../utils/entities/adminTool/div-help-content";
import { HelpContentService } from "../../services/help-content.service";
import { EnvProperties } from '../../utils/properties/env-properties';

import {Session} from '../../login/utils/helper.class';
import {LoginErrorCodes} from '../../login/utils/guardHelper.class';
import {HelperFunctions} from "../../utils/HelperFunctions.class";
import {Page} from "../../utils/entities/adminTool/page";

@Component({
    selector: 'new-div-help-content',
    templateUrl: 'new-div-help-content.component.html',
})

export class NewDivHelpContentComponent {

    @ViewChild(DivContentFormComponent)
    public formComponent : DivContentFormComponent;

    //private errorMessage : string = null;

    public communityPid: string;

    public pageId: string;
    public page: Page;

    public properties:EnvProperties = null;

    public showLoading: boolean = true;
    public errorMessage: string = '';
    public updateErrorMessage: string = '';

    constructor(
        private element: ElementRef,
        private route: ActivatedRoute,
        private router: Router,
        private _helpContentService: HelpContentService) {}

    ngOnInit() {
      this.route.data
        .subscribe((data: { envSpecific: EnvProperties }) => {
           this.properties = data.envSpecific;
           this.route.queryParams.subscribe(params => {
             HelperFunctions.scroll();

             this.communityPid = params['communityId'];
             this.pageId = params['pageId'];


             if(this.pageId) {
               this.getPage(this.pageId);
             } else {
               this.showLoading = false;
             }
           });
        });
    }

    private getPage(pageId: string) {
      this._helpContentService.getPageByPortal(pageId,this.properties.adminToolsAPIURL, this.communityPid).subscribe(
        page => {
          if(this.properties.adminToolsPortalType != page.portalType) {
            this.router.navigate(['../../'], { queryParams: { "communityId": this.communityPid}, relativeTo: this.route });
          } else {
            this.page = page;
            this.showLoading = false;
            console.info(this.page);
          }
        },
        error => this.handleError('System error retrieving page with id: '+pageId, error)
      );
    }

    public saveCustom() {
      if(!Session.isLoggedIn()){
        this.router.navigate(['/user-info'], { queryParams: { "errorCode": LoginErrorCodes.NOT_VALID, "redirectUrl":  this.router.url} });
      } else {
        if(this.formComponent.myForm.valid) {
            this.showLoading = true;
            this.updateErrorMessage = "";

            let divHelpContent : DivHelpContent = this.formComponent.myForm.value;

            this._helpContentService.insertOrUpdateDivHelpContent(divHelpContent, this.properties.adminToolsAPIURL, this.communityPid).subscribe(
                _ => {
                  if(this.pageId) {
                    this.router.navigate( ['../../'], { queryParams: { "communityId": this.communityPid, "pageId": this.pageId  } , relativeTo: this.route} );
                  } else {
                    this.router.navigate(['../../'], { queryParams: { "communityId": this.communityPid } , relativeTo: this.route} );
                  }
                  this.showLoading = false;
                },
                err => this.handleUpdateError('System error saving page content', err)
            );
        } else {
            this.errorMessage = "Please fill all required fields";
        }
      }
    }

    public cancelCustom() {
      if(this.pageId) {
        this.router.navigate( ['../../'], { queryParams: { "communityId": this.communityPid, "pageId": this.pageId  }, relativeTo: this.route } );
      } else {
        this.router.navigate(['../../'], { queryParams: { "communityId": this.communityPid }, relativeTo: this.route } );
      }
    }

    handleUpdateError(message: string, error) {
        this.errorMessage = message;
        console.log('Server responded: ' + error);
        this.showLoading = false;
    }

    handleError(message: string, error) {
      this.errorMessage = message;
      console.log('Server responded: ' + error);
      this.showLoading = false;
    }
}
