import { Component, ViewChild, OnInit, OnDestroy, ElementRef } from '@angular/core';
import { PageContentFormComponent } from "./page-help-content-form.component";
import {Subscriber, Subscription} from "rxjs";
import { HelpContentService } from "../../services/help-content.service";
import { PageHelpContent } from "../../utils/entities/adminTool/page-help-content";
import { ActivatedRoute, Router } from "@angular/router";
import { EnvProperties } from '../../utils/properties/env-properties';

import {Session} from '../../login/utils/helper.class';
import {LoginErrorCodes} from '../../login/utils/guardHelper.class';
import {HelperFunctions} from "../../utils/HelperFunctions.class";
import {Page} from "../../utils/entities/adminTool/page";
import {properties} from "../../../../environments/environment";

@Component({
    selector: 'edit-page-help-content',
    templateUrl: 'edit-page-help-content.component.html',
})

export class EditPageHelpContentComponent implements OnInit, OnDestroy{

    @ViewChild(PageContentFormComponent)
    public formComponent : PageContentFormComponent;

    public communityPid: string;

    public pageId: string;
    public page: Page;

    private subs: Subscription[]=[];

    private pageHelpContent: PageHelpContent;

    //private errorMessage : string = null;
    public properties:EnvProperties = null;

    public showLoading: boolean = true;
    public errorMessage: string = '';
    public updateErrorMessage: string = '';

    constructor(
        private element: ElementRef,
        private route: ActivatedRoute,
        private router: Router,
        private _helpContentService: HelpContentService) {}

    ngOnInit() {

       this.properties = properties;
       this.subs.push(this.route.queryParams.subscribe(params => {
         HelperFunctions.scroll();

         let pageContentId = params['pageContentId'];
         this.communityPid = params['communityId'];
         this.pageId = params['pageId'];

         if(!pageContentId) {
           this.router.navigate(['../../'], { queryParams: { "communityId": this.communityPid}, relativeTo: this.route});
         }

         this.getPageHelpContent(pageContentId);
       }));
    }
    ngOnDestroy() {
      this.subs.forEach(value => {
        if (value instanceof Subscriber) {
          value.unsubscribe();
        }
      });
    }

    handleError(message: string, error) {
        this.errorMessage = message;
        console.log('Server responded: ' + error);
        this.showLoading = false;
    }

    handleUpdateError(message: string, error) {
        this.updateErrorMessage = message;
        console.log('Server responded: ' + error);
        this.showLoading = false;
    }

    private getPage(pageId: string) {
      this.subs.push(this._helpContentService.getPageByPortal(pageId,this.properties.adminToolsAPIURL,  this.communityPid).subscribe(
        page => {
          if(this.properties.adminToolsPortalType != page.portalType) {
            this.router.navigate(['../../'], { queryParams: { "communityId": this.communityPid} , relativeTo: this.route});
          } else {
            this.page = page;
            this.formComponent.setPlacements(this.page);
            this.showLoading = false;
          }
        },
        error => this.handleError('System error retrieving page with id: '+pageId, error)
      ));
    }

    private getPageHelpContent(pageContentId: string) {
      if(!Session.isLoggedIn()){
        this.router.navigate(['/user-info'], { queryParams: { "errorCode": LoginErrorCodes.NOT_VALID, "redirectUrl": this.router.url} });
      } else {
        this.showLoading = true;
        this.errorMessage = "";
        this.updateErrorMessage = "";

        this.subs.push(this._helpContentService.getPageHelpContent(pageContentId as string, this.properties.adminToolsAPIURL, this.communityPid).subscribe(
          pageHelpContent => {
            if(this.pageId && this.pageId != pageHelpContent.page) {
              this.router.navigate(['../../'], { queryParams: { "communityId": this.communityPid} , relativeTo: this.route});
            } else if(this.pageId) {
              this.getPage(this.pageId);
            } else {
              this.getPage(<string>pageHelpContent.page);
            }
            this.updateForm(pageHelpContent);
            //this.showLoading = false;
          },
          error => this.handleError('System error retrieving page help content', error)));
      }
    }

    private updateForm(pageHelpContent : PageHelpContent) {
        this.pageHelpContent = pageHelpContent;
        this.formComponent.myForm.patchValue((pageHelpContent));
        // console.log("patching",pageHelpContent);
    }

  public saveCustom() {
      if(!Session.isLoggedIn()){
        this.router.navigate(['/user-info'], { queryParams: { "errorCode": LoginErrorCodes.NOT_VALID, "redirectUrl": this.router.url} });
      } else {
        if(this.formComponent.myForm.valid) {
            this.showLoading = true;
            this.updateErrorMessage = "";

            let pageHelpContent : PageHelpContent = this.formComponent.myForm.value;
          this.subs.push(this._helpContentService.updatePageHelpContent(pageHelpContent, this.properties.adminToolsAPIURL, this.communityPid).subscribe(
                _ => {
                  if(this.pageId) {
                    this.router.navigate( ['../../'], { queryParams: { "communityId": this.communityPid, "pageId": this.pageId  }, relativeTo: this.route } );
                  } else {
                    this.router.navigate(['../../'], { queryParams: { "communityId": this.communityPid}, relativeTo: this.route } );
                  }
                  this.showLoading = false;
                },
                err => this.handleUpdateError('System error updating page content', err)
            ));
        } else {
            this.errorMessage = "Please fill all required fields";
        }
      }
    }

  public cancelCustom() {
      this.errorMessage = "";
      this.updateErrorMessage = "";

      if(this.pageId) {
        this.router.navigate( ['../../'], { queryParams: { "communityId": this.communityPid, "pageId": this.pageId  }, relativeTo: this.route } );
      } else {
        this.router.navigate(['../../'], { queryParams: { "communityId": this.communityPid}, relativeTo: this.route } );
      }
    }
}
