import {Component, Input} from '@angular/core';

import { FetchResearchResults } from '../../utils/fetchEntitiesClasses/fetchResearchResults.class';

import { ErrorCodes} from '../../utils/properties/errorCodes';
import { RouterHelper } from '../../utils/routerHelper.class';
import { EnvProperties } from '../../utils/properties/env-properties';
import {AlertModal} from "../../utils/modal/alert";
import {properties} from "../../../../environments/environment";

@Component({
  selector: 'relatedDatasourcesTab',
  template: `
    <errorMessages  [status]="[fetchResults.searchUtils.status]"
                    [type]="'related content providers'" tab_error_class=true>
    </errorMessages>
    
    <div *ngIf="fetchResults.searchUtils.status == errorCodes.DONE && !loading">
      <div class="uk-text-muted uk-text-small uk-margin-bottom">
        *Only top 100 content providers that host research outcomes and are aggregated by {{collectedFromName}} are shown.
      </div>
      <no-load-paging *ngIf="results.length >pageSize"
                      [type]="'content providers'"
                      (pageChange)="updatePage($event)"
                      [page]="page" [pageSize]="pageSize"
                      [totalResults]="results.length">
      </no-load-paging>
    
      <table class="uk-table uk-table-striped">
        <thead>
        <tr>
          <th class="uk-text-center">Content Provider Name</th>
          <th *ngIf="fetchResults.results.length > 0 || fetchResults.searchUtils.status == errorCodes.ERROR"
              class="uk-text-center">
            Number of Research Outcomes
          </th>
        </tr>
        </thead>
        <tbody>
          <ng-container *ngIf="results">
            <ng-container *ngFor="let result of results; let i=index">
              <tr *ngIf="i>=(page-1)*pageSize && i<page*pageSize">
                <td class="uk-text-center">
                  <a [queryParams]="{datasourceId: result.id}" routerLinkActive="router-link-active" [routerLink]="dataProviderUrl"
                     (click)="onLinkClick()">
                    {{result.name}}
                  </a>
                </td>
                <td *ngIf="fetchResults.results.length > 0" class="uk-text-center">
                  <a
                    [queryParams]="routerHelper.createQueryParams(['f0', 'fv0', 'f1', 'fv1', 'f2', 'fv2', 'qf'], ['collectedfromdatasourceid', dataproviderId, 'resulthostingdatasourceid,or', dataproviderId, 'resulthostingdatasourceid', result.id, 'false'])"
                    routerLinkActive="router-link-active" [routerLink]="properties.searchLinkToAdvancedResults"
                    (click)="onLinkClick()">
                    {{result.count | number}}
                  </a>
                </td>
              </tr>
            </ng-container>
          </ng-container>
        </tbody>
      </table>
      <no-load-paging *ngIf="results.length >pageSize"
                      [type]="'content providers'"
                      (pageChange)="updatePage($event)"
                      [page]="page" [pageSize]="pageSize"
                      [totalResults]="results.length">
      </no-load-paging>
    </div>
  `
})

export class RelatedDatasourcesTabComponent {
    @Input() dataproviderId: string;
    // @Input() fetchPublications : FetchResearchResults;
    // @Input() fetchDatasets : FetchResearchResults;
    // @Input() fetchSoftware : FetchResearchResults;
    // @Input() fetchOrps: FetchResearchResults;
    @Input() fetchResults: FetchResearchResults;

    // true: preprocessing is not over
    @Input() loading: boolean = true;
    // Εvery content provider's id is a single key of a map
    //@Input() results: Map<string, {"name": string, "countPublications": string, "countDatasets": string, "countSoftware": string, "countOrps": string}>;
    @Input() results: {"id": string, "name": string, "count": number}[];
    @Input() properties:EnvProperties ;
    @Input() collectedFromName: string ="[no title available]";
    @Input() modal: AlertModal;

    public routerHelper:RouterHelper = new RouterHelper();
    public errorCodes:ErrorCodes = new ErrorCodes();

    public page: number = 1;
    public pageSize: number = 10;
    public dataProviderUrl;

    constructor () {}

    ngOnInit() {
      this.dataProviderUrl = properties.searchLinkToDataProvider.split('?')[0];
    }

    ngOnDestroy() {}

    totalPages(totalResults: number): number {
      let totalPages:any = totalResults/this.pageSize;
      if(!(Number.isInteger(totalPages))) {
          totalPages = (parseInt(totalPages, this.pageSize) + 1);
      }
      return totalPages;
    }

    updatePage($event) {
      this.page = $event.value;
    }

    public getKeys( map) {
      return Array.from(map.keys());
    }

    public onLinkClick() {
      if(this.modal) {
        this.modal.cancel();
      }
    }
}
