import {Component, EventEmitter, Input, Output, ViewChild} from '@angular/core';
import {ActivatedRoute} from '@angular/router';
import {AdvancedField, Filter} from './searchUtils/searchHelperClasses.class';
import {SearchDataprovidersService} from '../services/searchDataproviders.service';
import {ErrorCodes} from '../utils/properties/errorCodes';
import {ErrorMessagesComponent} from '../utils/errorMessages.component';
import {SearchFields} from '../utils/properties/searchFields';
import {SearchCustomFilter, SearchUtilsClass} from './searchUtils/searchUtils.class';

import {EnvProperties} from '../utils/properties/env-properties';
import {NewSearchPageComponent} from "./searchUtils/newSearchPage.component";
import {DatasourcesHelperClass} from "./searchUtils/datasourcesHelper.class";
import {properties} from "../../../environments/environment";


@Component({
    selector: 'search-dataproviders',
    template: `
 
    <new-search-page
      pageTitle="{{(simpleView?'':'Advanced ')}} Find research {{ pageTitle }}"
      entityType="dataprovider"
      type="content providers"
      [results]="results"
      [searchUtils]="searchUtils"
      [sortedByChanged]="searchUtils.sortBy" [resultsPerPageChanged]="searchUtils.size"
      [fieldIds]="fieldIds" [fieldIdsMap]="fieldIdsMap" [selectedFields]="selectedFields"
      [csvParams]="csvParams" csvPath="datasources"
      [simpleSearchLink]="simpleSearchLink" [advancedSearchLink]="advancedSearchLink"
      [disableForms]="disableForms"
      [disableRefineForms]="disableRefineForms"
      [loadPaging]="loadPaging"
      [oldTotalResults]="oldTotalResults"
      [openaireLink]=openaireLink
      [includeOnlyResultsAndFilter]="includeOnlyResultsAndFilter"
      [piwikSiteId]=piwikSiteId [hasPrefix]="hasPrefix"
      [searchFormClass]="type!='deposit'?'datasourcesSearchForm':''"
      [entitiesSelection]="type=='all'"  [showSwitchSearchLink]="showSwitchSearchLink"  
      [filters]="filters"
      [simpleView]="simpleView" formPlaceholderText="Search by name, description, subject..." 
      [showResultCount]="(type=='all' || type == 'deposit')" [showLastIndex]="type!='deposit'"
      [tableViewLink]="tableViewLink"
      [sort]="false" [showBreadcrumb]="showBreadcrumb" [basicMetaDescription]="metaDescription">

    </new-search-page>
<!--    [filters]="filters"-->

    `
 })

export class SearchDataProvidersComponent {
  private errorCodes: ErrorCodes;
  private errorMessages: ErrorMessagesComponent;
@Input() piwikSiteId = null;
@Input() customFilter:SearchCustomFilter= null;
@Input() tableViewLink;
  public results =[];
  public filters =[];
  public searchUtils:SearchUtilsClass = new SearchUtilsClass();
  public searchFields:SearchFields = new SearchFields();

  public fieldIds:  string[] = this.searchFields.DATASOURCE_ADVANCED_FIELDS;
  public fieldIdsMap= this.searchFields.DATASOURCE_FIELDS;
  public selectedFields:AdvancedField[] =  [];

  public resourcesQuery = "(oaftype exact datasource)";
  public csvParams: string;
  public disableForms: boolean = false;
  public disableRefineForms: boolean = false;
  public loadPaging: boolean = true;
  public oldTotalResults: number = 0;
  public pagingLimit: number = 0;
  public isPiwikEnabled;
  properties:EnvProperties;
  @Input() type: "all" | "registries" | "journals" | "compatible" | "deposit" = "all";
  public refineFields: string[];
  pageTitle;
  @ViewChild(NewSearchPageComponent) searchPage: NewSearchPageComponent;
  @Input() simpleView: boolean = true;
  @Input() simpleSearchLink: string = "";
  advancedSearchLink: string = "";
  @Input() hasPrefix: boolean = true;
  @Input() openaireLink: string;
  @Input() includeOnlyResultsAndFilter: boolean = false;
  @Output() searchPageUpdates = new EventEmitter();
  @Input() showSwitchSearchLink:boolean;
  @Input() showBreadcrumb: boolean = false;
  subs: any[] = [];
  searchResultsSub: any;
  searchFiltersSub: any;
  metaDescription = [];

  constructor (private route: ActivatedRoute, private _searchDataProvidersService: SearchDataprovidersService ) {
    this.results =[];
    this.errorCodes = new ErrorCodes();
    this.errorMessages = new ErrorMessagesComponent();
    this.searchUtils.status = this.errorCodes.LOADING;
  }
  ngOnInit() {
    this.refineFields = DatasourcesHelperClass.getrefineFields(this.type);
    this.pageTitle = DatasourcesHelperClass.getTitle(this.type);
    this.metaDescription = DatasourcesHelperClass.getDescription(this.type)
    if(this.showSwitchSearchLink == null){
      this.showSwitchSearchLink = (this.type == "all");
    }
    this.properties= properties;
    if (!this.simpleSearchLink) {
      this.simpleSearchLink = this.properties.searchLinkToDataProviders;
    }        this.advancedSearchLink = this.properties.searchLinkToAdvancedDataProviders;
    this.searchUtils.baseUrl = (this.simpleView)?this.simpleSearchLink:this.advancedSearchLink;
     this.pagingLimit = properties.pagingLimit;
     this.isPiwikEnabled = properties.enablePiwikTrack;

    let firstLoad = true;
    this.subs.push(this.route.queryParams.subscribe(params => {
     this.loadPaging = true;
     if(params['page'] && this.searchUtils.page != params['page']) {
       this.loadPaging = false;
       this.oldTotalResults = this.searchUtils.totalResults;
     }
     var refine = true;
      if (params['page'] != undefined && this.filters && !firstLoad && this.searchUtils.page != +params['page']) {
        refine = false;
      }

      if (params['size'] != undefined && this.filters && !firstLoad && this.searchUtils.size != params['size']) {
        refine = false;
      }

      let page = (params['page']=== undefined)?0:+params['page'];
      this.searchUtils.page = ( page < 1 ) ? 1 : page;

      this.searchUtils.size = (params['size']=== undefined)?10:+params['size'];
      if(this.searchUtils.size != 5 && this.searchUtils.size != 10 && this.searchUtils.size != 20 && this.searchUtils.size != 50) {
        this.searchUtils.size = 10;
      }

      this.searchPage.fieldIds = this.fieldIds;
      this.selectedFields =[];
      if(this.type == "deposit"){
        this.searchPage.keywordFields = this.searchFields.DEPOSIT_DATASOURCE_KEYWORD_FIELDS;
        this.searchPage.usedBy = "deposit";
      }
      // console.log(this.refineFields)
      this.searchPage.prepareSearchPage(this.fieldIds, this.selectedFields, this.refineFields, [], this.fieldIdsMap,this.customFilter,params, "dataprovider");
      if(refine && (this.type == "all" || this.type == "deposit")) {
        this._getFilters(this.searchPage.getSearchAPIQueryForAdvancedSearhFields(), this.searchUtils.page, 0, true, this.searchPage.getSearchAPIQueryForRefineFields(params, firstLoad));
      } else if(refine) { // static filters
        this.searchUtils.refineStatus = this.errorCodes.LOADING;
        this.filters = DatasourcesHelperClass.createFilters(this.type);
      } else {
        this.searchUtils.refineStatus = this.errorCodes.DONE;
      }
      this.getResults(this.searchPage.getSearchAPIQueryForAdvancedSearhFields(),  this.searchUtils.page, this.searchUtils.size, refine, this.searchPage.getSearchAPIQueryForRefineFields(params, firstLoad));
      firstLoad = false;
    }));
  }
  ngOnDestroy() {
    if(this.searchResultsSub) {
      this.searchResultsSub.unsubscribe();
    }
    if(this.searchFiltersSub) {
      this.searchFiltersSub.unsubscribe();
    }
    for (let sub of this.subs) {
      sub.unsubscribe();
    }
  }

  public _getFilters(parameters:string, page: number, size: number, refine: boolean, refineFieldsFilterQuery = null){
    if (page <= this.pagingLimit || this.searchUtils.refineStatus == this.errorCodes.LOADING) {
      this.searchUtils.refineStatus = this.errorCodes.LOADING;
      this.disableRefineForms = true;
      this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils});

      let datasourceQueryPrefix = DatasourcesHelperClass.getQueryPrefix(this.type);
      this.searchFiltersSub = this._searchDataProvidersService.advancedSearchDataproviders( datasourceQueryPrefix +(datasourceQueryPrefix.length > 0 && parameters.length > 0 ?' and (':'') + parameters +(datasourceQueryPrefix.length > 0 && parameters.length > 0 ?' ) ':''),  page, size, this.properties, (refine && (this.type=="all" || this.type == "deposit")) ? this.searchPage.getRefineFieldsQuery() : null, this.searchPage.getFields(), refineFieldsFilterQuery, (this.type == "deposit"))
      //.switchMap(
        .subscribe(
          data => {
            let totalResults = data[0];
            let filters = data[2];
            this.filtersReturned(refine, filters, totalResults, page);
          },
          err => {
            this.handleError("Error getting organizations: ", err);
            this.searchUtils.refineStatus = this.errorMessages.getErrorCode(err.status);

            this.disableRefineForms = false;
            this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils})
          }
        );
    }
  }

  public filtersReturned(refine: boolean, filters: Filter[], totalResults, page: number) {
    if (refine) {
      this.filters = this.searchPage.prepareFiltersToShow(filters, totalResults);
    }

    this.searchUtils.refineStatus = this.errorCodes.DONE;
    if(totalResults == 0) {
      this.searchUtils.refineStatus = this.errorCodes.NONE;
    }

    if (this.searchUtils.refineStatus == this.errorCodes.DONE) {
      // Page out of limit!!!
      let totalPages: any = totalResults / (this.searchUtils.size);
      if (!(Number.isInteger(totalPages))) {
        totalPages = (parseInt(totalPages, 10) + 1);
      }
      if (totalPages < page) {
        this.searchUtils.refineStatus = this.errorCodes.OUT_OF_BOUND;
      }
    }

    if(this.searchUtils.refineStatus != this.errorCodes.DONE && this.searchUtils.status != this.searchUtils.refineStatus) {
      if (this.searchResultsSub) {
        this.searchResultsSub.unsubscribe();
      }
      this.resultsReturned(refine, [], totalResults, page);
    }

    this.disableRefineForms = false;
    this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils})

  }

  public getResults(parameters:string, page: number, size: number, refine: boolean, refineFieldsFilterQuery = null){
    if(page > this.pagingLimit ) {
      size=0;
    }
    if(page <= this.pagingLimit  || this.searchUtils.status == this.errorCodes.LOADING) {
      // if(parameters!= null && parameters != ''  ) {
      //   this.csvParams ="&fq=( "+this.resourcesQuery + "and (" + parameters + "))";
      // }else{
      //   this.csvParams ="&fq="+this.resourcesQuery;
      // }
      // this.csvParams += (refineFieldsFilterQuery?refineFieldsFilterQuery:'');

      this.csvParams = (parameters ? ("&fq=("+parameters) : "") + (parameters ? ")" : "");
      this.csvParams += (refineFieldsFilterQuery ? refineFieldsFilterQuery : "");

      //var errorCodes:ErrorCodes = new ErrorCodes();
      this.searchUtils.status = this.errorCodes.LOADING;
      //this.searchPage.openLoading();
      this.disableForms = true;
      this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils});

      this.results = [];
      this.searchUtils.totalResults = 0;

      //console.info("Advanced Search for Content Providers: Execute search query "+parameters);
      let datasourceQueryPrefix = DatasourcesHelperClass.getQueryPrefix(this.type);
      //this._searchDataProvidersService.advancedSearchDataproviders( datasourceQueryPrefix +(datasourceQueryPrefix.length > 0 && parameters.length > 0 ?' and (':'') + parameters +(datasourceQueryPrefix.length > 0 && parameters.length > 0 ?' ) ':''),  page, size, this.properties, (refine && (this.type=="all" || this.type == "deposit")) ? this.searchPage.getRefineFieldsQuery() : null, this.searchPage.getFields(), refineFieldsFilterQuery, (this.type == "deposit")).subscribe(
      this.searchResultsSub = this._searchDataProvidersService.advancedSearchDataproviders( datasourceQueryPrefix +(datasourceQueryPrefix.length > 0 && parameters.length > 0 ?' and (':'') + parameters +(datasourceQueryPrefix.length > 0 && parameters.length > 0 ?' ) ':''),  page, size, this.properties, null, this.searchPage.getFields(), refineFieldsFilterQuery, (this.type == "deposit")).subscribe(
        data => {
          let totalResults = data[0];
          let results = data[1];
          if(refine && !(this.type == "all" || this.type == "deposit")){
            this.filtersReturned(refine, this.filters, totalResults, page);
          }
          this.resultsReturned(refine, results, totalResults, page);
          },
          err => {
              //console.log(err);
              this.handleError("Error getting content providers", err);
              this.searchUtils.status = this.errorMessages.getErrorCode(err.status);

              //TODO check erros (service not available, bad request)
              // if( ){
              //   this.searchUtils.status = errorCodes.ERROR;
              // }
              //var errorCodes:ErrorCodes = new ErrorCodes();
              //this.searchUtils.status = errorCodes.NOT_AVAILABLE;
              /*if(err.status == '404') {
                this.searchUtils.status = this.errorCodes.NOT_FOUND;
              } else if(err.status == '500') {
                this.searchUtils.status = this.errorCodes.ERROR;
              } else {
                this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
              }*/
              //this.searchPage.closeLoading();
              this.disableForms = false;
            this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils});
            this.searchPage.hideFilters = false;
          }
        );
      }
  }

  public resultsReturned(refine: boolean, results: any, totalResults, page: number) {
    this.searchUtils.totalResults = totalResults;
    this.results = results;
    if(!refine) {
      this.searchPage.buildPageURLParameters(this.filters, [], false);
    }

    this.searchPage.hideFilters = false;

    this.searchUtils.status = this.errorCodes.DONE;
    if (this.searchUtils.totalResults == 0) {
      this.searchUtils.status = this.errorCodes.NONE;
    }

    if (this.searchUtils.status == this.errorCodes.DONE) {
      // Page out of limit!!!
      let totalPages: any = this.searchUtils.totalResults / (this.searchUtils.size);
      if (!(Number.isInteger(totalPages))) {
        totalPages = (parseInt(totalPages, 10) + 1);
      }
      if (totalPages < page) {
        this.searchUtils.totalResults = 0;
        this.searchUtils.status = this.errorCodes.OUT_OF_BOUND;
      }
    }

    if(this.searchUtils.status != this.errorCodes.DONE && this.searchUtils.refineStatus != this.searchUtils.status) {
      if(this.searchFiltersSub) {
        this.searchFiltersSub.unsubscribe();
      }
      this.filtersReturned(refine, [], totalResults, page);
    }

    this.disableForms = false;
    this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils});

  }

  private handleError(message: string, error) {
    console.error("Content Providers advanced Search Page: "+message, error);
  }
}
