import {Component, EventEmitter, Input, Output, ViewChild} from '@angular/core';
import {ActivatedRoute} from '@angular/router';
import {AdvancedField, Filter} from './searchUtils/searchHelperClasses.class';
import {SearchOrganizationsService} from '../services/searchOrganizations.service';
import {ErrorCodes} from '../utils/properties/errorCodes';
import {ErrorMessagesComponent} from '../utils/errorMessages.component';
import {SearchFields} from '../utils/properties/searchFields';
import {SearchCustomFilter, SearchUtilsClass} from './searchUtils/searchUtils.class';
import {EnvProperties} from '../utils/properties/env-properties';
import {NewSearchPageComponent} from "./searchUtils/newSearchPage.component";
import {properties} from "../../../environments/environment";


@Component({
    selector: 'search-organizations',
    template: `
      <new-search-page
        pageTitle="{{(simpleView?'':'Advanced ')}} Search for {{ 'organizations' | titlecase }}"
        entityType="organization"
        type="organizations"
        [results]="results"
        [searchUtils]="searchUtils"
        [sortedByChanged]="searchUtils.sortBy" [resultsPerPageChanged]="searchUtils.size"
        [fieldIds]="fieldIds" [fieldIdsMap]="fieldIdsMap" [selectedFields]="selectedFields"
        [csvParams]="csvParams" csvPath="organizations"
        [simpleSearchLink]="simpleSearchLink" [advancedSearchLink]="advancedSearchLink"
        [disableForms]="disableForms"
        [disableRefineForms]="disableRefineForms"
        [loadPaging]="loadPaging"
        [oldTotalResults]="oldTotalResults"
        [openaireLink]=openaireLink
        [piwikSiteId]=piwikSiteId [hasPrefix]="hasPrefix"
        searchFormClass="organizationsSearchForm"
        [includeOnlyResultsAndFilter]="includeOnlyResultsAndFilter"
        [sort]="false"
        [filters]="filters"
        [simpleView]="simpleView" formPlaceholderText="Search by organization name..."
        [showSwitchSearchLink]="showSwitchSearchLink"
        [showBreadcrumb]="showBreadcrumb"
      >
      </new-search-page>
    `
 })

export class SearchOrganizationsComponent {
  private errorCodes: ErrorCodes;
  private errorMessages: ErrorMessagesComponent;
  properties:EnvProperties;
@Input() piwikSiteId = null;
  public results =[];
  public filters =[];
  public searchUtils:SearchUtilsClass = new SearchUtilsClass();
  public searchFields:SearchFields = new SearchFields();

  public fieldIds:  string[] = this.searchFields.ORGANIZATION_ADVANCED_FIELDS;
  public fieldIdsMap  = this.searchFields.ORGANIZATION_FIELDS;
  public selectedFields:AdvancedField[] =  [];
  public csvParams: string;
  public disableForms: boolean = false;
  public disableRefineForms: boolean = false;
  public loadPaging: boolean = true;
  public oldTotalResults: number = 0;
  public pagingLimit: number = 0;
  public isPiwikEnabled;

  @Input() customFilter:SearchCustomFilter= null;
  public refineFields: string[] = this.searchFields.ORGANIZATION_REFINE_FIELDS;
  @ViewChild(NewSearchPageComponent) searchPage: NewSearchPageComponent;
  @Input() simpleView: boolean = true;
  @Input() simpleSearchLink: string = "";
  advancedSearchLink: string = "";
  @Input() hasPrefix: boolean = true;
  @Input() openaireLink: string;
  @Input() includeOnlyResultsAndFilter: boolean = false;
  @Output() searchPageUpdates = new EventEmitter();
  @Input() showSwitchSearchLink:boolean = true;
  @Input() showBreadcrumb: boolean = false;
  public resourcesQuery = "(oaftype exact organization)";

  subs: any[] = [];
  searchResultsSub: any;
  searchFiltersSub: any;

  constructor (private route: ActivatedRoute, private _searchOrganizationsService: SearchOrganizationsService ) {
    this.results =[];
    this.errorCodes = new ErrorCodes();
    this.errorMessages = new ErrorMessagesComponent();
    this.searchUtils.status = this.errorCodes.LOADING;



  }
  ngOnInit() {

    this.properties= properties;
    if (!this.simpleSearchLink) {
      this.simpleSearchLink = this.properties.searchLinkToOrganizations;
    }
    this.advancedSearchLink = this.properties.searchLinkToAdvancedOrganizations;
    this.searchUtils.baseUrl = (this.simpleView)?this.simpleSearchLink:this.advancedSearchLink;
    this.pagingLimit = properties.pagingLimit;
    this.isPiwikEnabled = properties.enablePiwikTrack;

    this.searchUtils.status = this.errorCodes.LOADING;
    var firstLoad = true;

    this.subs.push(this.route.queryParams.subscribe(params => {
      this.loadPaging = true;
      if(params['page'] && this.searchUtils.page != params['page']) {
        this.loadPaging = false;
        this.oldTotalResults = this.searchUtils.totalResults;
      }
      var refine = true;
      if (params['page'] != undefined && this.filters && !firstLoad && this.searchUtils.page != +params['page']) {
        refine = false;
      }

      if (params['size'] != undefined && this.filters && !firstLoad && this.searchUtils.size != params['size']) {
        refine = false;
      }

      let page = (params['page']=== undefined)?1:+params['page'];
      this.searchUtils.page = ( page <= 0 ) ? 1 : page;

      this.searchUtils.size = (params['size']=== undefined)?10:+params['size'];
      if(this.searchUtils.size != 5 && this.searchUtils.size != 10 && this.searchUtils.size != 20 && this.searchUtils.size != 50) {
        this.searchUtils.size = 10;
      }

      this.searchPage.fieldIds = this.fieldIds;
      this.selectedFields =[];
      // this.searchPage.selectedFields = this.selectedFields;
      // this.searchPage.fieldIdsMap = this.fieldIdsMap;
      // this.searchPage.customFilter = this.customFilter;
      // this.searchPage.getSelectedFiltersFromUrl(params);
      this.searchPage.prepareSearchPage(this.fieldIds, this.selectedFields, this.refineFields, [], this.fieldIdsMap,this.customFilter,params, "organization");
      if(refine) {
        this._getFilters(this.searchPage.getSearchAPIQueryForAdvancedSearhFields(),  this.searchUtils.page, 0, true, this.searchPage.getSearchAPIQueryForRefineFields(params, firstLoad));
      } else {
        this.searchUtils.refineStatus = this.errorCodes.DONE;
      }
      this.getResults(this.searchPage.getSearchAPIQueryForAdvancedSearhFields(),  this.searchUtils.page, this.searchUtils.size, refine, this.searchPage.getSearchAPIQueryForRefineFields(params, firstLoad));
      firstLoad = false;
    }));
  }
  ngOnDestroy() {
    if(this.searchResultsSub) {
      this.searchResultsSub.unsubscribe();
    }
    if(this.searchFiltersSub) {
      this.searchFiltersSub.unsubscribe();
    }
    for (let sub of this.subs) {
      sub.unsubscribe();
    }
  }

  public _getFilters(parameters:string, page: number, size: number, refine: boolean, refineFieldsFilterQuery = null){
    if (page <= this.pagingLimit || this.searchUtils.refineStatus == this.errorCodes.LOADING) {
      this.searchUtils.refineStatus = this.errorCodes.LOADING;
      this.disableRefineForms = true;
      this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils});
      this.searchFiltersSub = this._searchOrganizationsService.advancedSearchOrganizations(parameters, page, size, this.properties,(refine) ? this.searchPage.getRefineFieldsQuery() : null, this.searchPage.getFields(), refineFieldsFilterQuery)
      //.switchMap(
        .subscribe(
          data => {
            let totalResults = data[0];
            let filters = data[2];
            this.filtersReturned(refine, filters, totalResults, page);
          },
          err => {
            this.handleError("Error getting organizations: ", err);
            this.searchUtils.refineStatus = this.errorMessages.getErrorCode(err.status);

            this.disableRefineForms = false;
            this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils})
          }
        );
    }
  }

  public filtersReturned(refine: boolean, filters: Filter[], totalResults, page: number) {
    if (refine) {
      this.filters = this.searchPage.prepareFiltersToShow(filters, totalResults);
    }

    this.searchUtils.refineStatus = this.errorCodes.DONE;
    if(totalResults == 0) {
      this.searchUtils.refineStatus = this.errorCodes.NONE;
    }

    if (this.searchUtils.refineStatus == this.errorCodes.DONE) {
      // Page out of limit!!!
      let totalPages: any = totalResults / (this.searchUtils.size);
      if (!(Number.isInteger(totalPages))) {
        totalPages = (parseInt(totalPages, 10) + 1);
      }
      if (totalPages < page) {
        this.searchUtils.refineStatus = this.errorCodes.OUT_OF_BOUND;
      }
    }

    if(this.searchUtils.refineStatus != this.errorCodes.DONE && this.searchUtils.status != this.searchUtils.refineStatus) {
      if (this.searchResultsSub) {
        this.searchResultsSub.unsubscribe();
      }
      this.resultsReturned(refine, [], totalResults, page);
    }

    this.disableRefineForms = false;
    this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils})

  }

  public getResults(parameters:string, page: number, size: number, refine: boolean, refineFieldsFilterQuery = null){
    if(page > this.pagingLimit) {
      size=0;
    }
    if(page <= this.pagingLimit || this.searchUtils.status == this.errorCodes.LOADING) {
      // if(parameters!= null && parameters != ''  ) {
      //   this.csvParams ="&fq=( "+this.resourcesQuery + "and (" + parameters + "))";
      // }else{
      //   this.csvParams ="&fq="+this.resourcesQuery;
      // }

      this.csvParams = (parameters ? ("&fq=("+parameters) : "") + (parameters ? ")" : "");
      this.csvParams += (refineFieldsFilterQuery ? refineFieldsFilterQuery : "");

      //var errorCodes:ErrorCodes = new ErrorCodes();
      this.searchUtils.status = this.errorCodes.LOADING;
      //this.searchPage.openLoading();
      this.disableForms = true;
      this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils});

      this.results = [];
      this.searchUtils.totalResults = 0;

      //console.info("Advanced Search for Organizations: Execute search query "+parameters);
       //this._searchOrganizationsService.advancedSearchOrganizations(parameters, page, size, this.properties,(refine) ? this.searchPage.getRefineFieldsQuery() : null, this.searchPage.getFields(), refineFieldsFilterQuery).subscribe(
      this.searchResultsSub = this._searchOrganizationsService.advancedSearchOrganizations(parameters, page, size, this.properties, null, this.searchPage.getFields(), refineFieldsFilterQuery).subscribe(
        data => {
          let totalResults = data[0];
          let results = data[1];
          this.resultsReturned(refine, results, totalResults, page);
        },
        err => {
            //console.log(err);
            this.handleError("Error getting organizations", err);
            this.searchUtils.status = this.errorMessages.getErrorCode(err.status);
            //this.searchUtils.totalResults = 0;
            //TODO check erros (service not available, bad request)
            // if( ){
            //   this.searchUtils.status = errorCodes.ERROR;
            // }
            //var errorCodes:ErrorCodes = new ErrorCodes();
            //this.searchUtils.status = errorCodes.NOT_AVAILABLE;
            /*if(err.status == '404') {
              this.searchUtils.status = this.errorCodes.NOT_FOUND;
            } else if(err.status == '500') {
              this.searchUtils.status = this.errorCodes.ERROR;
            } else {
              this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
            }*/

            //this.searchPage.closeLoading();
            this.disableForms = false;
          this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils});
          this.searchPage.hideFilters = false;
        }
      );
    }
  }

  public resultsReturned(refine: boolean, results: any, totalResults, page: number) {
    this.searchUtils.totalResults = totalResults;
    this.results = results;
    if(!refine) {
      this.searchPage.buildPageURLParameters(this.filters, [], false);
    }

    this.searchPage.hideFilters = false;

    this.searchUtils.status = this.errorCodes.DONE;
    if (this.searchUtils.totalResults == 0) {
      this.searchUtils.status = this.errorCodes.NONE;
    }

    if (this.searchUtils.status == this.errorCodes.DONE) {
      // Page out of limit!!!
      let totalPages: any = this.searchUtils.totalResults / (this.searchUtils.size);
      if (!(Number.isInteger(totalPages))) {
        totalPages = (parseInt(totalPages, 10) + 1);
      }
      if (totalPages < page) {
        this.searchUtils.totalResults = 0;
        this.searchUtils.status = this.errorCodes.OUT_OF_BOUND;
      }
    }

    if(this.searchUtils.status != this.errorCodes.DONE && this.searchUtils.refineStatus != this.searchUtils.status) {
      if(this.searchFiltersSub) {
        this.searchFiltersSub.unsubscribe();
      }
      this.filtersReturned(refine, [], totalResults, page);
    }

    this.disableForms = false;
    this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils})

  }

  private handleError(message: string, error) {
    console.error("Organizations advanced Search Page: "+message, error);
  }
}
