import {Component, EventEmitter, Input, OnChanges, Output, SimpleChanges, ViewChild} from '@angular/core';
import {ActivatedRoute} from '@angular/router';
import {AdvancedField, Filter} from './searchUtils/searchHelperClasses.class';
import {SearchResearchResultsService} from '../services/searchResearchResults.service';
import {ErrorCodes} from '../utils/properties/errorCodes';
import {ErrorMessagesComponent} from '../utils/errorMessages.component';
import {SearchFields} from '../utils/properties/searchFields';
import {SearchCustomFilter, SearchUtilsClass} from './searchUtils/searchUtils.class';
import {EnvProperties} from '../utils/properties/env-properties';
import {NewSearchPageComponent} from "./searchUtils/newSearchPage.component";
import {RangeFilter} from "../utils/rangeFilter/rangeFilterHelperClasses.class";
import {properties} from "../../../environments/environment";
import {ContextsService} from "../claims/claim-utils/service/contexts.service";

@Component({
  selector: 'search-research-results',
  template: `
    <new-search-page
        pageTitle="{{(simpleView?'':'Advanced ')}} Search for {{ getEntityName(resultType, true, true) | titlecase }}"
        [entityType]="resultType"
        [type]="getEntityName(resultType, true, true)"
        [results]="results"
        [searchUtils]="searchUtils"
        [sortedByChanged]="searchUtils.sortBy" [resultsPerPageChanged]="searchUtils.size"
        [fieldIds]="fieldIds" [fieldIdsMap]="fieldIdsMap" [selectedFields]="selectedFields"
        [csvParams]="csvParams" [csvPath]="getEntityName(resultType, true, false)"
        [simpleSearchLink]="simpleSearchLink" [advancedSearchLink]="advancedSearchLink"
        [disableForms]="disableForms"
        [disableRefineForms]="disableRefineForms"
        [loadPaging]="loadPaging"
        [oldTotalResults]="oldTotalResults"
        [openaireLink]=openaireLink
        [piwikSiteId]=piwikSiteId [hasPrefix]="hasPrefix"
        searchFormClass="publicationsSearchForm"
        [filters]="filters" [quickFilter]="quickFilter"
        [rangeFilters]="rangeFilters" [rangeFields]="rangeFields"
        [simpleView]="simpleView" formPlaceholderText="Search by title, author, abstract, DOI, orcid..."
        [includeOnlyResultsAndFilter]="includeOnlyResultsAndFilter" [showBreadcrumb]="showBreadcrumb"
        [showSwitchSearchLink]="showSwitchSearchLink"
        [stickyForm]="stickyForm"
    >
    </new-search-page>
  
  `
})

export class SearchResearchResultsComponent {
  @Input() resultType: string = "result";
  
  @Input() simpleSearchLink: string = "";
  advancedSearchLink: string = "";
  
  private errorCodes: ErrorCodes;
  private errorMessages: ErrorMessagesComponent;
  @Input() piwikSiteId = properties.piwikSiteId;
  @Input() hasPrefix: boolean = true;
  public results = [];
  public filters = [];
  public rangeFilters: RangeFilter[] = [];
  
  public searchUtils: SearchUtilsClass = new SearchUtilsClass();
  public searchFields: SearchFields = new SearchFields();
  
  public fieldIds: string[] = this.searchFields.RESULT_ADVANCED_FIELDS;
  public fieldIdsMap = this.searchFields.RESULT_FIELDS;
  public rangeFields: string[][] = this.searchFields.RESULT_RANGE_FIELDS;
  public selectedFields: AdvancedField[] = [];
  public resourcesQuery = "((oaftype exact result) and (resulttypeid exact " + this.resultType + "))";
  public csvParams: string;
  public disableForms: boolean = false;
  public disableRefineForms: boolean = false;
  public loadPaging: boolean = true;
  public oldTotalResults: number = 0;
  @Input() openaireLink: string;
  @Input() customFilter: SearchCustomFilter = null;
  public pagingLimit: number = 0;
  public isPiwikEnabled;
  properties: EnvProperties = properties;
  public refineFields: string[] = this.searchFields.RESULT_REFINE_FIELDS;
  @ViewChild(NewSearchPageComponent) searchPage: NewSearchPageComponent;
  @Input() simpleView: boolean = true;
  quickFilter: { filter: Filter, selected: boolean, filterId: string, value: string } = {
    filter: null,
    selected: true,
    filterId: "resultbestaccessright",
    value: "Open Access"
  };
  @Input() includeOnlyResultsAndFilter: boolean = false;
  @Input() showBreadcrumb: boolean = false;
  @Output() searchPageUpdates = new EventEmitter();
  @Input() showSwitchSearchLink: boolean = true;
  subs: any[] = [];
  searchResultsSub: any;
  searchFiltersSub: any;
  @Input() stickyForm:boolean = false;

  private timeoutId;
  private promise: Promise<boolean> = null;
  private publicCommunities = null;

  constructor(private route: ActivatedRoute, private _searchResearchResultsService: SearchResearchResultsService,
              private _contextService: ContextsService) {
    this.results = [];
    this.errorCodes = new ErrorCodes();
    this.errorMessages = new ErrorMessagesComponent();
    this.searchUtils.status = this.errorCodes.LOADING;
    this.searchUtils.refineStatus = this.errorCodes.LOADING;
    this.getPublicCommunities();
  }
  
  ngOnInit() {
    //TODO add checks about which result types are enabled!
    this.pagingLimit = this.properties.pagingLimit;
    this.isPiwikEnabled = this.properties.enablePiwikTrack;
    
    if (!this.simpleSearchLink) {
      this.simpleSearchLink = this.properties.searchLinkToResults;
    }
    this.advancedSearchLink = this.properties.searchLinkToAdvancedResults;
    this.searchUtils.baseUrl = (this.simpleView) ? this.simpleSearchLink : this.advancedSearchLink;
    
    this.searchUtils.status = this.errorCodes.LOADING;
    var firstLoad = true;
    this.subs.push(this.route.queryParams.subscribe(params => {
    //this.subs.push(this.route.queryParams.switchMap(params => {
      this.loadPaging = true;
      if (params['page'] && this.searchUtils.page != params['page']) {
        this.loadPaging = false;
        this.oldTotalResults = this.searchUtils.totalResults;
      }
      var refine = true;
      if (params['page'] != undefined && this.filters && !firstLoad && this.searchUtils.page != +params['page']) {
        refine = false;
      }

      if (params['sortBy'] != undefined && this.filters && !firstLoad && this.searchUtils.sortBy != params['sortBy']) {
        refine = false;
      }

      if (params['size'] != undefined && this.filters && !firstLoad && this.searchUtils.size != params['size']) {
        refine = false;
      }

      let page = (params['page'] === undefined) ? 1 : +params['page'];
      this.searchUtils.page = (page <= 0) ? 1 : page;
      
      this.searchUtils.size = (params['size'] === undefined) ? 10 : +params['size'];
      if (this.searchUtils.size != 5 && this.searchUtils.size != 10 && this.searchUtils.size != 20 && this.searchUtils.size != 50) {
        this.searchUtils.size = 10;
      }
      this.searchUtils.sortBy = (params['sortBy']) ? params['sortBy'] : '';
      if (this.searchUtils.sortBy && this.searchUtils.sortBy != "resultdateofacceptance,descending" && this.searchUtils.sortBy != "resultdateofacceptance,ascending") {
        this.searchUtils.sortBy = "";
      }
      this.selectedFields = [];

      this.searchPage.prepareSearchPage(this.fieldIds, this.selectedFields, this.refineFields, this.rangeFields, this.fieldIdsMap,this.customFilter,params, this.resultType, this.quickFilter);
      if(refine) {
        this._getFilters(this.searchPage.getSearchAPIQueryForAdvancedSearhFields(), this.searchUtils.page, 0, "", true, this.searchPage.getSearchAPIQueryForRangeFields(params)+this.searchPage.getSearchAPIQueryForRefineFields(params, firstLoad));
      } else {
        this.searchUtils.refineStatus = this.errorCodes.DONE;
      }
      this._getResults(this.searchPage.getSearchAPIQueryForAdvancedSearhFields(), this.searchUtils.page, this.searchUtils.size, this.searchUtils.sortBy, refine, this.searchPage.getSearchAPIQueryForRangeFields(params) + this.searchPage.getSearchAPIQueryForRefineFields(params, firstLoad));
      firstLoad = false;
    }));
  }
  
  ngOnDestroy() {
    if(this.searchResultsSub) {
      this.searchResultsSub.unsubscribe();
    }
    if(this.searchFiltersSub) {
      this.searchFiltersSub.unsubscribe();
    }
    for (let sub of this.subs) {
      sub.unsubscribe();
    }
  }

  public getPublicCommunities() {
    this.promise = new Promise<any>(resolve => {
      this._contextService.getPublicCommunitiesByState().subscribe(
        data => {
          if(this.publicCommunities == null) {
            this.publicCommunities = data.map(value => value.id);
          }
          resolve();
        },
        error => {
          this.handleError("Error getting communities status", error);
          resolve();
        });
    });
  }

  racePromiseWithTimeout() {
    const timeoutPromise = new Promise((_, reject) => {
      this.timeoutId = setTimeout(() => {
        this.publicCommunities = [];
        reject(new Error('Request timed out'));
      }, 300);
    });

    return Promise.race([this.promise, timeoutPromise]);
  }


  public _getFilters(parameters: string, page: number, size: number, sortBy: string, refine: boolean, refineFieldsFilterQuery = null) {
      if (page <= this.pagingLimit || this.searchUtils.refineStatus == this.errorCodes.LOADING) {
        this.searchUtils.refineStatus = this.errorCodes.LOADING;
        this.disableRefineForms = true;
        this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils});
        this.searchFiltersSub = this._searchResearchResultsService.advancedSearchResults(this.resultType, parameters, page, size, sortBy, this.properties, (refine) ? this.searchPage.getRefineFieldsQuery() : null, this.searchPage.getFields(), refineFieldsFilterQuery)
        // this.subs.push(this._searchResearchResultsService.advancedSearchResults(this.resultType, parameters, page, size, sortBy, this.properties, (refine) ? this.searchPage.getRefineFieldsQuery() : null, this.searchPage.getFields(), refineFieldsFilterQuery)
          //.switchMap(
          .subscribe(
            data => {
              let totalResults = data[0];
              let filters = data[2];

              // if (refine) {
              //   this.filters = this.searchPage.prepareFiltersToShow(filters, totalResults);
              //   this.rangeFilters = this.searchPage.prepareRangeFiltersToShow();
              // }
              //
              // this.searchUtils.refineStatus = this.errorCodes.DONE;
              //
              // this.disableRefineForms = false;
              this.filtersReturned(refine, filters, totalResults, page);
              this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils})
            },
            err => {
              this.handleError("Error getting " + this.getEntityName(this.resultType, true, true), err);
              this.searchUtils.refineStatus = this.errorMessages.getErrorCode(err.status);

              this.disableRefineForms = false;
              this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils})
            }
          );//);
      }
  }

    public async filtersReturned(refine: boolean, filters: Filter[], totalResults, page: number) {
      if(!this.timeoutId) {
        try {
          await this.racePromiseWithTimeout();
        } catch (error) {
          this.handleError("Getting communities status was over 300ms - timeout", error);
        } finally {
          clearTimeout(this.timeoutId);
        }
      }

      let communitiesFilter = filters.find(filters => filters.filterId == "community");
      if(communitiesFilter && this.publicCommunities && this.publicCommunities.length > 0) {
        communitiesFilter.values = communitiesFilter.values.filter(value => {
          let valueId = "";
          if(value.id) {
            let idArray = value.id.split("||");
            if(idArray) {
              valueId = idArray[0];
            }
          }
          //for(let publicCom of this.publicCommunities) {
            //if(valueId && publicCom['id'] == valueId) {
          if(valueId && this.publicCommunities.includes(valueId)) {
              return true;
            //}
          }
        });
      }

      if (refine) {
        this.filters = this.searchPage.prepareFiltersToShow(filters, totalResults);
        this.rangeFilters = this.searchPage.prepareRangeFiltersToShow();
      }

      this.searchUtils.refineStatus = this.errorCodes.DONE;
      if(totalResults == 0) {
        this.searchUtils.refineStatus = this.errorCodes.NONE;
      }

      if (this.searchUtils.refineStatus == this.errorCodes.DONE) {
        // Page out of limit!!!
        let totalPages: any = totalResults / (this.searchUtils.size);
        if (!(Number.isInteger(totalPages))) {
          totalPages = (parseInt(totalPages, 10) + 1);
        }
        if (totalPages < page) {
          this.searchUtils.refineStatus = this.errorCodes.OUT_OF_BOUND;
        }
      }

      if(this.searchUtils.refineStatus != this.errorCodes.DONE && this.searchUtils.status != this.searchUtils.refineStatus) {
        if (this.searchResultsSub) {
          this.searchResultsSub.unsubscribe();
        }
        this.resultsReturned(refine, [], totalResults, page);
      }

      this.disableRefineForms = false;
    }

  public _getResults(parameters: string, page: number, size: number, sortBy: string, refine: boolean, refineFieldsFilterQuery = null) {
    if (page > this.pagingLimit) {
      size = 0;
    }
    if (page <= this.pagingLimit || this.searchUtils.status == this.errorCodes.LOADING) {
      // if (parameters != null && parameters != '') {
      //   this.csvParams = "&fq=(" + this.resourcesQuery + " and (" + parameters + "))";
      // } else {
      //   this.csvParams = "&fq=" + this.resourcesQuery;
      // }
      this.csvParams = (parameters ? ("&fq=(" + parameters) : "") + (parameters ? ")" : "");
      this.csvParams += (refineFieldsFilterQuery ? refineFieldsFilterQuery : "");
      
      this.searchUtils.status = this.errorCodes.LOADING;
      this.disableForms = true;
      this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils});
      this.results = [];
      this.searchUtils.totalResults = 0;
      //this.subs.push(this._searchResearchResultsService.advancedSearchResults(this.resultType, parameters, page, size, sortBy, this.properties, (refine) ? this.searchPage.getRefineFieldsQuery() : null, this.searchPage.getFields(), refineFieldsFilterQuery)
      this.searchResultsSub = this._searchResearchResultsService.advancedSearchResults(this.resultType, parameters, page, size, sortBy, this.properties, null, this.searchPage.getFields(), refineFieldsFilterQuery)
      //this.subs.push(this._searchResearchResultsService.advancedSearchResults(this.resultType, parameters, page, size, sortBy, this.properties, null, this.searchPage.getFields(), refineFieldsFilterQuery)
        .subscribe(
        data => {
          let totalResults = data[0];
          let results = data[1];

          // this.searchUtils.totalResults = totalResults;
          // this.results = results;
          // // if (refine) {
          // //   this.filters = this.searchPage.prepareFiltersToShow(data[2],this.searchUtils.totalResults);
          // //   this.rangeFilters = this.searchPage.prepareRangeFiltersToShow();
          // // } else {
          // if(!refine) {
          //   this.searchPage.buildPageURLParameters(this.filters, this.rangeFilters, false);
          // }
          //
          // this.searchPage.hideFilters = false;
          //
          // this.searchUtils.status = this.errorCodes.DONE;
          // if (this.searchUtils.totalResults == 0) {
          //   this.searchUtils.status = this.errorCodes.NONE;
          // }
          // this.disableForms = false;
          this.resultsReturned(refine, results, totalResults, page);
        },
        err => {
          this.handleError("Error getting " + this.getEntityName(this.resultType, true, true), err);
          this.searchUtils.status = this.errorMessages.getErrorCode(err.status);

          //TODO check erros (service not available, bad request)
          // if( ){
          //   this.searchUtils.status = ErrorCodes.ERROR;
          // }
          //var errorCodes:ErrorCodes = new ErrorCodes();
          //this.searchUtils.status = errorCodes.NOT_AVAILABLE;
          /*if(err.status == '404') {
            this.searchUtils.status = this.errorCodes.NOT_FOUND;
          } else if(err.status == '500') {
            this.searchUtils.status = this.errorCodes.ERROR;
          } else {
            this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
          }*/

          //this.searchPage.closeLoading();
          this.disableForms = false;
          this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils})
          this.searchPage.hideFilters = false;
        }
      );
    }
  }

  public resultsReturned(refine: boolean, results: any, totalResults, page: number) {
    this.searchUtils.totalResults = totalResults;
    this.results = results;
    if(!refine) {
      this.searchPage.buildPageURLParameters(this.filters, this.rangeFilters, false);
    }

    this.searchPage.hideFilters = false;

    this.searchUtils.status = this.errorCodes.DONE;
    if (this.searchUtils.totalResults == 0) {
      this.searchUtils.status = this.errorCodes.NONE;
    }

    if (this.searchUtils.status == this.errorCodes.DONE) {
      // Page out of limit!!!
      let totalPages: any = this.searchUtils.totalResults / (this.searchUtils.size);
      if (!(Number.isInteger(totalPages))) {
        totalPages = (parseInt(totalPages, 10) + 1);
      }
      if (totalPages < page) {
        this.searchUtils.totalResults = 0;
        this.searchUtils.status = this.errorCodes.OUT_OF_BOUND;
      }
    }

    if(this.searchUtils.status != this.errorCodes.DONE && this.searchUtils.refineStatus != this.searchUtils.status) {
      if(this.searchFiltersSub) {
        this.searchFiltersSub.unsubscribe();
      }
      this.filtersReturned(refine, [], totalResults, page);
    }

    this.disableForms = false;
    this.searchPageUpdates.emit({disableForms: this.disableForms, disableRefineForms: this.disableRefineForms, searchUtils: this.searchUtils});
  }
  
  private handleError(message: string, error) {
    console.error(this.getEntityName(this.resultType, true, true) + " advanced Search Page: " + message, error);
  }
  
  public getEntityName(entityType: string, plural: boolean, full: boolean): string {
    if (entityType == "publication") {
      return "publication" + (plural ? "s" : "");
    } else if (entityType == "dataset") {
      return (full ? "research data" : ("dataset" + (plural ? "s" : "")));
    } else if (entityType == "software") {
      return "software";
    } else if (entityType == "other") {
      return (full ? ("other research product" + (plural ? "s" : "")) : "other");
    } else if (entityType == "result") {
      return (full ? ("research outcome" + (plural ? "s" : "")) : "result");
    }
  }
}
